@@ -84,8 +84,8 @@ void debug(const char* fmt, ...) {
 
 /* Structure storing a pair of communities along with their dQ values */
 typedef struct s_igraph_i_fastgreedy_commpair {
-    long int first;       /* first member of the community pair */
-    long int second;      /* second member of the community pair */
+    igraph_integer_t first;  /* first member of the community pair */
+    igraph_integer_t second; /* second member of the community pair */
     igraph_real_t *dq;    /* pointer to a member of the dq vector storing the */
     /* increase in modularity achieved when joining */
     struct s_igraph_i_fastgreedy_commpair *opposite;
@@ -101,7 +101,7 @@ typedef struct {
 
 /* Global community list structure */
 typedef struct {
-    long int no_of_communities, n;  /* number of communities, number of vertices */
+    igraph_integer_t no_of_communities, n; /* number of communities, number of vertices */
     igraph_i_fastgreedy_community* e;     /* list of communities */
     igraph_i_fastgreedy_community** heap; /* heap of communities */
     igraph_integer_t *heapindex; /* heap index to speed up lookup by community idx */
@@ -110,16 +110,15 @@ typedef struct {
 /* Scans the community neighborhood list for the new maximal dq value.
  * Returns 1 if the maximum is different from the previous one,
  * 0 otherwise. */
-static int igraph_i_fastgreedy_community_rescan_max(
-        igraph_i_fastgreedy_community* comm) {
-    long int i, n;
+static igraph_bool_t igraph_i_fastgreedy_community_rescan_max(igraph_i_fastgreedy_community* comm) {
+    igraph_integer_t i, n;
     igraph_i_fastgreedy_commpair *p, *best;
     igraph_real_t bestdq, currdq;
 
     n = igraph_vector_ptr_size(&comm->neis);
     if (n == 0) {
         comm->maxdq = 0;
-        return 1;
+        return true;
     }
 
     best = (igraph_i_fastgreedy_commpair*)VECTOR(comm->neis)[0];
@@ -135,32 +134,32 @@ static int igraph_i_fastgreedy_community_rescan_max(
 
     if (best != comm->maxdq) {
         comm->maxdq = best;
-        return 1;
+        return true;
     } else {
-        return 0;
+        return false;
     }
 }
 
 /* Destroys the global community list object */
 static void igraph_i_fastgreedy_community_list_destroy(
         igraph_i_fastgreedy_community_list* list) {
-    long int i;
+    igraph_integer_t i;
     for (i = 0; i < list->n; i++) {
         igraph_vector_ptr_destroy(&list->e[i].neis);
     }
-    igraph_Free(list->e);
+    IGRAPH_FREE(list->e);
     if (list->heapindex != 0) {
-        igraph_Free(list->heapindex);
+        IGRAPH_FREE(list->heapindex);
     }
     if (list->heap != 0) {
-        igraph_Free(list->heap);
+        IGRAPH_FREE(list->heap);
     }
 }
 
 /* Community list heap maintenance: sift down */
 static void igraph_i_fastgreedy_community_list_sift_down(
-        igraph_i_fastgreedy_community_list* list, long int idx) {
-    long int root, child, c1, c2;
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t idx) {
+    igraph_integer_t root, child, c1, c2;
     igraph_i_fastgreedy_community* dummy;
     igraph_integer_t dummy2;
     igraph_i_fastgreedy_community** heap = list->heap;
@@ -194,8 +193,8 @@ static void igraph_i_fastgreedy_community_list_sift_down(
 
 /* Community list heap maintenance: sift up */
 static void igraph_i_fastgreedy_community_list_sift_up(
-        igraph_i_fastgreedy_community_list* list, long int idx) {
-    long int root, parent, c1, c2;
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t idx) {
+    igraph_integer_t root, parent, c1, c2;
     igraph_i_fastgreedy_community* dummy;
     igraph_integer_t dummy2;
     igraph_i_fastgreedy_community** heap = list->heap;
@@ -226,7 +225,7 @@ static void igraph_i_fastgreedy_community_list_sift_up(
 /* Builds the community heap for the first time */
 static void igraph_i_fastgreedy_community_list_build_heap(
         igraph_i_fastgreedy_community_list* list) {
-    long int i;
+    igraph_integer_t i;
     for (i = list->no_of_communities / 2 - 1; i >= 0; i--) {
         igraph_i_fastgreedy_community_list_sift_down(list, i);
     }
@@ -240,20 +239,20 @@ static void igraph_i_fastgreedy_community_list_build_heap(
 /*
 static void igraph_i_fastgreedy_community_list_dump_heap(
         igraph_i_fastgreedy_community_list* list) {
-    long int i;
+    igraph_integer_t i;
     debug("Heap:\n");
     for (i = 0; i < list->no_of_communities; i++) {
         debug("(%ld, %p, %p)", i, list->heap[i],
               list->heap[i]->maxdq);
         if (list->heap[i]->maxdq) {
-            debug(" (%ld, %ld, %.7f)", list->heap[i]->maxdq->first,
+            debug(" (%" IGRAPH_PRId ", %" IGRAPH_PRId ", %.7f)", list->heap[i]->maxdq->first,
                   list->heap[i]->maxdq->second, *list->heap[i]->maxdq->dq);
         }
         debug("\n");
     }
     debug("Heap index:\n");
     for (i = 0; i < list->no_of_communities; i++) {
-        debug("%ld ", (long)list->heapindex[i]);
+        debug("%" IGRAPH_PRId " ", list->heapindex[i]);
     }
     debug("\nEND\n");
 }
@@ -264,12 +263,12 @@ static void igraph_i_fastgreedy_community_list_dump_heap(
 /*
 static void igraph_i_fastgreedy_community_list_check_heap(
         igraph_i_fastgreedy_community_list* list) {
-    long int i;
+    igraph_integer_t i;
     for (i = 0; i < list->no_of_communities / 2; i++) {
         if ((2 * i + 1 < list->no_of_communities && *list->heap[i]->maxdq->dq < *list->heap[2 * i + 1]->maxdq->dq) ||
             (2 * i + 2 < list->no_of_communities && *list->heap[i]->maxdq->dq < *list->heap[2 * i + 2]->maxdq->dq)) {
             IGRAPH_WARNING("Heap property violated");
-            debug("Position: %ld, %ld and %ld\n", i, 2 * i + 1, 2 * i + 2);
+            debug("Position: %" IGRAPH_PRId ", %" IGRAPH_PRId " and %" IGRAPH_PRId "\n", i, 2 * i + 1, 2 * i + 2);
             igraph_i_fastgreedy_community_list_dump_heap(list);
         }
     }
@@ -278,13 +277,13 @@ static void igraph_i_fastgreedy_community_list_check_heap(
 
 /* Removes a given element from the heap */
 static void igraph_i_fastgreedy_community_list_remove(
-        igraph_i_fastgreedy_community_list* list, long int idx) {
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t idx) {
     igraph_real_t old;
-    long int commidx;
+    igraph_integer_t commidx;
 
     /* First adjust the index */
     commidx = list->heap[list->no_of_communities - 1]->maxdq->first;
-    list->heapindex[commidx] = (igraph_integer_t) idx;
+    list->heapindex[commidx] = idx;
     commidx = list->heap[idx]->maxdq->first;
     list->heapindex[commidx] = -1;
 
@@ -304,8 +303,8 @@ static void igraph_i_fastgreedy_community_list_remove(
 /* Removes a given element from the heap when there are no more neighbors
  * for it (comm->maxdq is NULL) */
 static void igraph_i_fastgreedy_community_list_remove2(
-        igraph_i_fastgreedy_community_list* list, long int idx, long int comm) {
-    long int i;
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t idx, igraph_integer_t comm) {
+    igraph_integer_t i;
 
     if (idx == list->no_of_communities - 1) {
         /* We removed the rightmost element on the bottom level, no problem,
@@ -317,7 +316,7 @@ static void igraph_i_fastgreedy_community_list_remove2(
 
     /* First adjust the index */
     i = list->heap[list->no_of_communities - 1]->maxdq->first;
-    list->heapindex[i] = (igraph_integer_t) idx;
+    list->heapindex[i] = idx;
     list->heapindex[comm] = -1;
 
     /* Now remove the element */
@@ -333,9 +332,9 @@ static void igraph_i_fastgreedy_community_list_remove2(
 /* Removes the pair belonging to community k from the neighborhood list
  * of community c (that is, clist[c]) and recalculates maxdq */
 static void igraph_i_fastgreedy_community_remove_nei(
-        igraph_i_fastgreedy_community_list* list, long int c, long int k) {
-    long int i, n;
-    igraph_bool_t rescan = 0;
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t c, igraph_integer_t k) {
+    igraph_integer_t i, n;
+    igraph_bool_t rescan = false;
     igraph_i_fastgreedy_commpair *p;
     igraph_i_fastgreedy_community *comm;
     igraph_real_t olddq;
@@ -367,7 +366,7 @@ static void igraph_i_fastgreedy_community_remove_nei(
             } else {
                 /* no more neighbors for this community. we should remove this
                  * community from the heap and restore the heap property */
-                debug("REMOVING (NO MORE NEIS): %ld\n", i);
+                debug("REMOVING (NO MORE NEIS): %" IGRAPH_PRId "\n", i);
                 igraph_i_fastgreedy_community_list_remove2(list, i, c);
             }
         }
@@ -378,20 +377,22 @@ static void igraph_i_fastgreedy_community_remove_nei(
  * `second` field */
 static int igraph_i_fastgreedy_commpair_cmp(const void* p1, const void* p2) {
     igraph_i_fastgreedy_commpair *cp1, *cp2;
+    igraph_integer_t diff;
     cp1 = *(igraph_i_fastgreedy_commpair**)p1;
     cp2 = *(igraph_i_fastgreedy_commpair**)p2;
-    return (int) (cp1->second - cp2->second);
+    diff = cp1->second - cp2->second;
+    return (diff < 0) ? -1 : (diff > 0) ? 1 : 0;
 }
 
 /* Sorts the neighbor list of the community with the given index, optionally
  * optimizing the process if we know that the list is nearly sorted and only
  * a given pair is in the wrong place. */
 static void igraph_i_fastgreedy_community_sort_neighbors_of(
-        igraph_i_fastgreedy_community_list* list, long int index,
+        igraph_i_fastgreedy_community_list* list, igraph_integer_t index,
         igraph_i_fastgreedy_commpair* changed_pair) {
     igraph_vector_ptr_t* vec;
-    long int i, n;
-    igraph_bool_t can_skip_sort = 0;
+    igraph_integer_t i, n;
+    igraph_bool_t can_skip_sort = false;
     igraph_i_fastgreedy_commpair *other_pair;
 
     vec = &list->e[index].neis;
@@ -458,10 +459,11 @@ static void igraph_i_fastgreedy_community_sort_neighbors_of(
  * of the community list clist to newdq and restores the heap property
  * in community c if necessary. Returns 1 if the maximum in the row had
  * to be updated, zero otherwise */
-static int igraph_i_fastgreedy_community_update_dq(
+static igraph_bool_t igraph_i_fastgreedy_community_update_dq(
         igraph_i_fastgreedy_community_list* list,
         igraph_i_fastgreedy_commpair* p, igraph_real_t newdq) {
-    long int i, j, to, from;
+
+    igraph_integer_t i, j, to, from;
     igraph_real_t olddq;
     igraph_i_fastgreedy_community *comm_to, *comm_from;
     to = p->first; from = p->second;
@@ -593,8 +595,8 @@ static int igraph_i_fastgreedy_community_update_dq(
  *    weights are expected to be non-negative.
  * \param merges Pointer to an initialized matrix or \c NULL, the result of the
  *    computation is stored here. The matrix has two columns and each
- *    merge corresponds to one merge, the ids of the two merged
- *    components are stored. The component ids are numbered from zero and
+ *    merge corresponds to one merge, the IDs of the two merged
+ *    components are stored. The component IDs are numbered from zero and
  *    the first \c n components are the individual vertices, \c n is
  *    the number of vertices in the graph. Component \c n is created
  *    in the first merge, component <code>n+1</code> in the second merge, etc.
@@ -620,24 +622,25 @@ static int igraph_i_fastgreedy_community_update_dq(
  *
  * \example examples/simple/igraph_community_fastgreedy.c
  */
-int igraph_community_fastgreedy(const igraph_t *graph,
+igraph_error_t igraph_community_fastgreedy(const igraph_t *graph,
                                 const igraph_vector_t *weights,
-                                igraph_matrix_t *merges,
+                                igraph_matrix_int_t *merges,
                                 igraph_vector_t *modularity,
-                                igraph_vector_t *membership) {
-    long int no_of_edges, no_of_nodes, no_of_joins, total_joins;
-    long int i, j, k, n, m, from, to, dummy, best_no_of_joins;
+                                igraph_vector_int_t *membership) {
+    igraph_integer_t no_of_edges, no_of_nodes, no_of_joins, total_joins;
+    igraph_integer_t i, j, k, n, m, from, to, dummy, best_no_of_joins;
     igraph_integer_t ffrom, fto;
     igraph_eit_t edgeit;
     igraph_i_fastgreedy_commpair *pairs, *p1, *p2;
     igraph_i_fastgreedy_community_list communities;
     igraph_vector_t a;
+    igraph_vector_int_t degrees;
     igraph_real_t q, *dq, bestq, weight_sum, loop_weight_sum;
     igraph_bool_t has_multiple;
-    igraph_matrix_t merges_local;
+    igraph_matrix_int_t merges_local;
 
-    /*long int join_order[] = { 16,5, 5,6, 6,0, 4,0, 10,0, 26,29, 29,33, 23,33, 27,33, 25,24, 24,31, 12,3, 21,1, 30,8, 8,32, 9,2, 17,1, 11,0, 7,3, 3,2, 13,2, 1,2, 28,31, 31,33, 22,32, 18,32, 20,32, 32,33, 15,33, 14,33, 0,19, 19,2, -1,-1 };*/
-    /*long int join_order[] = { 43,42, 42,41, 44,41, 41,36, 35,36, 37,36, 36,29, 38,29, 34,29, 39,29, 33,29, 40,29, 32,29, 14,29, 30,29, 31,29, 6,18, 18,4, 23,4, 21,4, 19,4, 27,4, 20,4, 22,4, 26,4, 25,4, 24,4, 17,4, 0,13, 13,2, 1,2, 11,2, 8,2, 5,2, 3,2, 10,2, 9,2, 7,2, 2,28, 28,15, 12,15, 29,16, 4,15, -1,-1 };*/
+    /*igraph_integer_t join_order[] = { 16,5, 5,6, 6,0, 4,0, 10,0, 26,29, 29,33, 23,33, 27,33, 25,24, 24,31, 12,3, 21,1, 30,8, 8,32, 9,2, 17,1, 11,0, 7,3, 3,2, 13,2, 1,2, 28,31, 31,33, 22,32, 18,32, 20,32, 32,33, 15,33, 14,33, 0,19, 19,2, -1,-1 };*/
+    /*igraph_integer_t join_order[] = { 43,42, 42,41, 44,41, 41,36, 35,36, 37,36, 36,29, 38,29, 34,29, 39,29, 33,29, 40,29, 32,29, 14,29, 30,29, 31,29, 6,18, 18,4, 23,4, 21,4, 19,4, 27,4, 20,4, 22,4, 26,4, 25,4, 24,4, 17,4, 0,13, 13,2, 1,2, 11,2, 8,2, 5,2, 3,2, 10,2, 9,2, 7,2, 2,28, 28,15, 12,15, 29,16, 4,15, -1,-1 };*/
 
     no_of_nodes = igraph_vcount(graph);
     no_of_edges = igraph_ecount(graph);
@@ -648,7 +651,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
 
     total_joins = no_of_nodes > 0 ? no_of_nodes - 1 : 0;
 
-    if (weights != 0) {
+    if (weights) {
         if (igraph_vector_size(weights) != no_of_edges) {
             IGRAPH_ERROR("Length of weight vector must agree with number of edges.", IGRAPH_EINVAL);
         }
@@ -674,14 +677,14 @@ int igraph_community_fastgreedy(const igraph_t *graph,
     if (membership != 0 && merges == 0) {
         /* We need the merge matrix because the user wants the membership
          * vector, so we allocate one on our own */
-        IGRAPH_CHECK(igraph_matrix_init(&merges_local, total_joins, 2));
-        IGRAPH_FINALLY(igraph_matrix_destroy, &merges_local);
+        IGRAPH_CHECK(igraph_matrix_int_init(&merges_local, total_joins, 2));
+        IGRAPH_FINALLY(igraph_matrix_int_destroy, &merges_local);
         merges = &merges_local;
     }
 
     if (merges != 0) {
-        IGRAPH_CHECK(igraph_matrix_resize(merges, total_joins, 2));
-        igraph_matrix_null(merges);
+        IGRAPH_CHECK(igraph_matrix_int_resize(merges, total_joins, 2));
+        igraph_matrix_int_null(merges);
     }
 
     if (modularity != 0) {
@@ -693,62 +696,68 @@ int igraph_community_fastgreedy(const igraph_t *graph,
     if (weights) {
         debug("Calculating weighted degrees\n");
         for (i = 0; i < no_of_edges; i++) {
-            VECTOR(a)[(long int)IGRAPH_FROM(graph, i)] += VECTOR(*weights)[i];
-            VECTOR(a)[(long int)IGRAPH_TO(graph, i)] += VECTOR(*weights)[i];
+            VECTOR(a)[IGRAPH_FROM(graph, i)] += VECTOR(*weights)[i];
+            VECTOR(a)[IGRAPH_TO(graph, i)] += VECTOR(*weights)[i];
         }
     } else {
         debug("Calculating degrees\n");
-        IGRAPH_CHECK(igraph_degree(graph, &a, igraph_vss_all(), IGRAPH_ALL, 1));
+        IGRAPH_VECTOR_INT_INIT_FINALLY(&degrees, no_of_nodes);
+        IGRAPH_CHECK(igraph_degree(graph, &degrees, igraph_vss_all(), IGRAPH_ALL, 1));
+        for (i = 0; i < no_of_nodes; i++) {
+            VECTOR(a)[i] = VECTOR(degrees)[i];
+        }
+        igraph_vector_int_destroy(&degrees);
+        IGRAPH_FINALLY_CLEAN(1);
     }
 
     /* Create list of communities */
     debug("Creating community list\n");
     communities.n = no_of_nodes;
     communities.no_of_communities = no_of_nodes;
-    communities.e = (igraph_i_fastgreedy_community*)calloc((size_t) no_of_nodes, sizeof(igraph_i_fastgreedy_community));
+    communities.e = IGRAPH_CALLOC(no_of_nodes, igraph_i_fastgreedy_community);
     if (communities.e == 0) {
-        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
     IGRAPH_FINALLY(igraph_free, communities.e);
-    communities.heap = (igraph_i_fastgreedy_community**)calloc((size_t) no_of_nodes, sizeof(igraph_i_fastgreedy_community*));
+    communities.heap = IGRAPH_CALLOC(no_of_nodes, igraph_i_fastgreedy_community*);
     if (communities.heap == 0) {
-        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
     IGRAPH_FINALLY(igraph_free, communities.heap);
-    communities.heapindex = (igraph_integer_t*)calloc((size_t)no_of_nodes, sizeof(igraph_integer_t));
+    communities.heapindex = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
     if (communities.heapindex == 0) {
-        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
     IGRAPH_FINALLY_CLEAN(2);
     IGRAPH_FINALLY(igraph_i_fastgreedy_community_list_destroy, &communities);
     for (i = 0; i < no_of_nodes; i++) {
-        igraph_vector_ptr_init(&communities.e[i].neis, 0);
-        communities.e[i].id = (igraph_integer_t) i;
+        IGRAPH_CHECK(igraph_vector_ptr_init(&communities.e[i].neis, 0));
+        communities.e[i].id = i;
         communities.e[i].size = 1;
     }
 
     /* Create list of community pairs from edges */
     debug("Allocating dq vector\n");
-    dq = (igraph_real_t*)calloc((size_t) no_of_edges, sizeof(igraph_real_t));
+    dq = IGRAPH_CALLOC(no_of_edges, igraph_real_t);
     if (dq == 0) {
-        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
     IGRAPH_FINALLY(igraph_free, dq);
     debug("Creating community pair list\n");
-    IGRAPH_CHECK(igraph_eit_create(graph, igraph_ess_all(0), &edgeit));
+    IGRAPH_CHECK(igraph_eit_create(graph, igraph_ess_all(IGRAPH_EDGEORDER_ID), &edgeit));
     IGRAPH_FINALLY(igraph_eit_destroy, &edgeit);
-    pairs = (igraph_i_fastgreedy_commpair*)calloc(2 * (size_t) no_of_edges, sizeof(igraph_i_fastgreedy_commpair));
+    pairs = IGRAPH_CALLOC(2 * no_of_edges, igraph_i_fastgreedy_commpair);
     if (pairs == 0) {
-        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+        IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
     IGRAPH_FINALLY(igraph_free, pairs);
     loop_weight_sum = 0;
     for (i = 0, j = 0; !IGRAPH_EIT_END(edgeit); i += 2, j++, IGRAPH_EIT_NEXT(edgeit)) {
-        long int eidx = IGRAPH_EIT_GET(edgeit);
-        igraph_edge(graph, (igraph_integer_t) eidx, &ffrom, &fto);
+        igraph_integer_t eidx = IGRAPH_EIT_GET(edgeit);
+        igraph_edge(graph, eidx, &ffrom, &fto);
 
         /* Create the pairs themselves */
-        from = (long int)ffrom; to = (long int)fto;
+        from = ffrom; to = fto;
         if (from == to) {
             loop_weight_sum += weights ? 2 * VECTOR(*weights)[eidx] : 2;
             continue;
@@ -771,8 +780,8 @@ int igraph_community_fastgreedy(const igraph_t *graph,
         pairs[i + 1].dq = pairs[i].dq;
         pairs[i + 1].opposite = &pairs[i];
         /* Link the pair to the communities */
-        igraph_vector_ptr_push_back(&communities.e[from].neis, &pairs[i]);
-        igraph_vector_ptr_push_back(&communities.e[to].neis, &pairs[i + 1]);
+        IGRAPH_CHECK(igraph_vector_ptr_push_back(&communities.e[from].neis, &pairs[i]));
+        IGRAPH_CHECK(igraph_vector_ptr_push_back(&communities.e[to].neis, &pairs[i + 1]));
         /* Update maximums */
         if (communities.e[from].maxdq == 0 || *communities.e[from].maxdq->dq < *pairs[i].dq) {
             communities.e[from].maxdq = &pairs[i];
@@ -792,7 +801,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
          * the heap (to avoid maxdq == 0) */
         if (communities.e[i].maxdq != 0) {
             communities.heap[j] = &communities.e[i];
-            communities.heapindex[i] = (igraph_integer_t) j;
+            communities.heapindex[i] = j;
             j++;
         } else {
             communities.heapindex[i] = -1;
@@ -841,7 +850,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
 
         /* Some debug info if needed */
         /* igraph_i_fastgreedy_community_list_check_heap(&communities); */
-#ifdef DEBUG
+#ifdef IGRAPH_FASTCOMM_DEBUG
         debug("===========================================\n");
         for (i = 0; i < communities.n; i++) {
             if (communities.e[i].maxdq == 0) {
@@ -875,7 +884,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
         debug("Q[%ld] = %.7f\tdQ = %.7f\t |H| = %ld\n",
               no_of_joins, q, *communities.heap[0]->maxdq->dq, no_of_nodes - no_of_joins - 1);
 
-        /* DEBUG */
+        /* IGRAPH_FASTCOMM_DEBUG */
         /* from=join_order[no_of_joins*2]; to=join_order[no_of_joins*2+1];
         if (to == -1) break;
         for (i=0; i<igraph_vector_ptr_size(&communities.e[to].neis); i++) {
@@ -897,13 +906,13 @@ int igraph_community_fastgreedy(const igraph_t *graph,
         while (i < n && j < m) {
             p1 = (igraph_i_fastgreedy_commpair*)VECTOR(communities.e[to].neis)[i];
             p2 = (igraph_i_fastgreedy_commpair*)VECTOR(communities.e[from].neis)[j];
-            debug("Pairs: %ld-%ld and %ld-%ld\n", p1->first, p1->second,
+            debug("Pairs: %" IGRAPH_PRId "-%" IGRAPH_PRId " and %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p1->first, p1->second,
                   p2->first, p2->second);
             if (p1->second < p2->second) {
                 /* Considering p1 from now on */
-                debug("    Considering: %ld-%ld\n", p1->first, p1->second);
+                debug("    Considering: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p1->first, p1->second);
                 if (p1->second == from) {
-                    debug("    WILL REMOVE: %ld-%ld\n", to, from);
+                    debug("    WILL REMOVE: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", to, from);
                 } else {
                     /* chain, case 1 */
                     debug("    CHAIN(1): %ld-%ld %ld, now=%.7f, adding=%.7f, newdq(%ld,%ld)=%.7f\n",
@@ -913,7 +922,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
                 i++;
             } else if (p1->second == p2->second) {
                 /* p1->first, p1->second and p2->first form a triangle */
-                debug("    Considering: %ld-%ld and %ld-%ld\n", p1->first, p1->second,
+                debug("    Considering: %" IGRAPH_PRId "-%" IGRAPH_PRId " and %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p1->first, p1->second,
                       p2->first, p2->second);
                 /* Update dq value */
                 debug("    TRIANGLE: %ld-%ld-%ld, now=%.7f, adding=%.7f, newdq(%ld,%ld)=%.7f\n",
@@ -923,9 +932,9 @@ int igraph_community_fastgreedy(const igraph_t *graph,
                 i++;
                 j++;
             } else {
-                debug("    Considering: %ld-%ld\n", p2->first, p2->second);
+                debug("    Considering: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p2->first, p2->second);
                 if (p2->second == to) {
-                    debug("    WILL REMOVE: %ld-%ld\n", p2->second, p2->first);
+                    debug("    WILL REMOVE: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p2->second, p2->first);
                 } else {
                     /* chain, case 2 */
                     debug("    CHAIN(2): %ld %ld-%ld, newdq(%ld,%ld)=%.7f\n",
@@ -960,7 +969,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
         while (i < n) {
             p1 = (igraph_i_fastgreedy_commpair*)VECTOR(communities.e[to].neis)[i];
             if (p1->second == from) {
-                debug("    WILL REMOVE: %ld-%ld\n", p1->first, from);
+                debug("    WILL REMOVE: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", p1->first, from);
             } else {
                 /* chain, case 1 */
                 debug("    CHAIN(1): %ld-%ld %ld, now=%.7f, adding=%.7f, newdq(%ld,%ld)=%.7f\n",
@@ -996,7 +1005,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
 
         /* Now, remove community `from` from the neighbors of community `to` */
         if (communities.no_of_communities > 2) {
-            debug("    REMOVING: %ld-%ld\n", to, from);
+            debug("    REMOVING: %" IGRAPH_PRId "-%" IGRAPH_PRId "\n", to, from);
             igraph_i_fastgreedy_community_remove_nei(&communities, to, from);
             i = igraph_i_fastgreedy_community_list_find_in_heap(&communities, from);
             igraph_i_fastgreedy_community_list_remove(&communities, i);
@@ -1015,7 +1024,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
         if (merges) {
             MATRIX(*merges, no_of_joins, 0) = communities.e[to].id;
             MATRIX(*merges, no_of_joins, 1) = communities.e[from].id;
-            communities.e[to].id = (igraph_integer_t) (no_of_nodes + no_of_joins);
+            communities.e[to].id = no_of_nodes + no_of_joins;
         }
 
         /* Update vector a */
@@ -1029,47 +1038,56 @@ int igraph_community_fastgreedy(const igraph_t *graph,
      * smallest decrease in modularity every step. Now we're simply deleting
      * the excess rows from the merge matrix */
     if (no_of_joins < total_joins) {
-        long int *ivec;
-        ivec = igraph_Calloc(igraph_matrix_nrow(merges), long int);
+        igraph_integer_t *ivec;
+        igraph_integer_t merges_nrow = igraph_matrix_int_nrow(merges);
+        ivec = IGRAPH_CALLOC(merges_nrow, igraph_integer_t);
         if (ivec == 0) {
-            IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM);
+            IGRAPH_ERROR("Insufficient memory for fast greedy community detection.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
         }
         IGRAPH_FINALLY(igraph_free, ivec);
         for (i = 0; i < no_of_joins; i++) {
             ivec[i] = i + 1;
         }
-        igraph_matrix_permdelete_rows(merges, ivec, total_joins - no_of_joins);
-        igraph_Free(ivec);
+        igraph_matrix_int_permdelete_rows(merges, ivec, total_joins - no_of_joins);
+        IGRAPH_FREE(ivec);
         IGRAPH_FINALLY_CLEAN(1);
     }
     IGRAPH_PROGRESS("Fast greedy community detection", 100.0, 0);
 
     if (modularity) {
         VECTOR(*modularity)[no_of_joins] = q;
-        igraph_vector_resize(modularity, no_of_joins + 1);
+        IGRAPH_CHECK(igraph_vector_resize(modularity, no_of_joins + 1));
+    }
+
+    /* Internally, the algorithm does not create NaN values.
+     * If the graph has no edges, the final modularity will be zero.
+     * We change this to NaN for consistency. */
+    if (modularity && no_of_edges == 0) {
+        IGRAPH_ASSERT(no_of_joins == 0);
+        VECTOR(*modularity)[0] = IGRAPH_NAN;
     }
 
     debug("Freeing memory\n");
-    igraph_Free(pairs);
-    igraph_Free(dq);
+    IGRAPH_FREE(pairs);
+    IGRAPH_FREE(dq);
     igraph_i_fastgreedy_community_list_destroy(&communities);
     igraph_vector_destroy(&a);
     IGRAPH_FINALLY_CLEAN(4);
 
     if (membership) {
         IGRAPH_CHECK(igraph_community_to_membership(merges,
-                     (igraph_integer_t) no_of_nodes,
-                     /*steps=*/ (igraph_integer_t) best_no_of_joins,
+                     no_of_nodes,
+                     /*steps=*/ best_no_of_joins,
                      membership,
                      /*csize=*/ 0));
     }
 
     if (merges == &merges_local) {
-        igraph_matrix_destroy(&merges_local);
+        igraph_matrix_int_destroy(&merges_local);
         IGRAPH_FINALLY_CLEAN(1);
     }
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 #ifdef IGRAPH_FASTCOMM_DEBUG
