@@ -22,9 +22,54 @@
 */
 
 #include "igraph_lapack.h"
-
 #include "linalg/lapack_internal.h"
 
+#include <limits.h>
+
+#define BASE_FORTRAN_INT
+#include "igraph_pmt.h"
+#include "igraph_vector_type.h"
+#include "igraph_vector_pmt.h"
+#include "../core/vector.pmt"
+#include "igraph_pmt_off.h"
+#undef BASE_FORTRAN_INT
+
+/* Converts a Fortran integer vector to an igraph vector */
+static igraph_error_t igraph_vector_int_update_from_fortran(
+    igraph_vector_int_t* vec, const igraph_vector_fortran_int_t* fortran_vec
+) {
+    igraph_integer_t size = igraph_vector_fortran_int_size(fortran_vec);
+
+    IGRAPH_CHECK(igraph_vector_int_resize(vec, size));
+
+    for (igraph_integer_t i = 0; i < size; i++) {
+        VECTOR(*vec)[i] = VECTOR(*fortran_vec)[i];
+    }
+
+    return IGRAPH_SUCCESS;
+}
+
+/* Allocates a Fortran integer vector from the contents of an igraph vector */
+static igraph_error_t igraph_vector_int_copy_to_fortran(
+    const igraph_vector_int_t* vec, igraph_vector_fortran_int_t* fortran_vec
+) {
+    igraph_integer_t i, size = igraph_vector_int_size(vec);
+
+    IGRAPH_CHECK(igraph_vector_fortran_int_resize(fortran_vec, size));
+
+    for (i = 0; i < size; i++) {
+        if (VECTOR(*vec)[i] > INT_MAX) {
+            IGRAPH_ERROR(
+                "Overflow error while copying an igraph integer vector to a "
+                "Fortran integer vector.", IGRAPH_EOVERFLOW
+            );
+        }
+        VECTOR(*fortran_vec)[i] = (int) VECTOR(*vec)[i];
+    }
+
+    return IGRAPH_SUCCESS;
+}
+
 /**
  * \function igraph_lapack_dgetrf
  * \brief LU factorization of a general M-by-N matrix.
@@ -42,7 +87,7 @@
  *      unless it is a null pointer. Row \c i of the matrix was
  *      interchanged with row <code>ipiv[i]</code>.
  * \param info LAPACK error code. Zero on successful exit. If its value is
- *      a positive number i, it indicates that U(i,i) is exactly zero. 
+ *      a positive number i, it indicates that U(i,i) is exactly zero.
  *      The factorization has been
  *      completed, but the factor U is exactly singular, and division
  *      by zero will occur if it is used to solve a system of
@@ -53,22 +98,30 @@
  * Time complexity: TODO.
  */
 
-int igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
+igraph_error_t igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
                          int *info) {
-    int m = (int) igraph_matrix_nrow(a);
-    int n = (int) igraph_matrix_ncol(a);
-    int lda = m > 0 ? m : 1;
-    igraph_vector_int_t *myipiv = ipiv, vipiv;
+    int m;
+    int n;
+    size_t num_elts;
+    int lda;
+    igraph_vector_fortran_int_t vipiv;
 
-    if (!ipiv) {
-        IGRAPH_CHECK(igraph_vector_int_init(&vipiv, m < n ? m : n));
-        IGRAPH_FINALLY(igraph_vector_int_destroy, &vipiv);
-        myipiv = &vipiv;
-    } else {
-        IGRAPH_CHECK(igraph_vector_int_resize(ipiv, m < n ? m : n));
+    if (igraph_matrix_nrow(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
+    if (igraph_matrix_ncol(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of columns in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
     }
 
-    igraphdgetrf_(&m, &n, VECTOR(a->data), &lda, VECTOR(*myipiv), info);
+    m = (int) igraph_matrix_nrow(a);
+    n = (int) igraph_matrix_ncol(a);
+    num_elts = m < n ? m : n;
+    lda = m > 0 ? m : 1;
+
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&vipiv, num_elts));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &vipiv);
+
+    igraphdgetrf_(&m, &n, VECTOR(a->data), &lda, VECTOR(vipiv), info);
 
     if (*info > 0) {
         IGRAPH_WARNING("LU: factor is exactly singular.");
@@ -98,12 +151,14 @@ int igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
         }
     }
 
-    if (!ipiv) {
-        igraph_vector_int_destroy(&vipiv);
-        IGRAPH_FINALLY_CLEAN(1);
+    if (ipiv) {
+        IGRAPH_CHECK(igraph_vector_int_update_from_fortran(ipiv, &vipiv));
     }
 
-    return 0;
+    igraph_vector_fortran_int_destroy(&vipiv);
+    IGRAPH_FINALLY_CLEAN(1);
+
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -120,8 +175,8 @@ int igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
  *      factorization A = P*L*U. L is expected to be unitriangular,
  *      diagonal entries are those of U. If A is singular, no warning or
  *      error wil be given and random output will be returned.
- * \param ipiv An integer vector, the pivot indices from \ref
- *      igraph_lapack_dgetrf() must be given here. Row \c i of A was
+ * \param ipiv An integer vector, the pivot indices from
+ *      \ref igraph_lapack_dgetrf() must be given here. Row \c i of A was
  *      interchanged with row <code>ipiv[i]</code>.
  * \param b The right hand side matrix must be given here. The solution
             will also be placed here.
@@ -130,14 +185,27 @@ int igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
  * Time complexity: TODO.
  */
 
-int igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix_t *a,
+igraph_error_t igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix_t *a,
                          const igraph_vector_int_t *ipiv, igraph_matrix_t *b) {
     char trans = transpose ? 'T' : 'N';
-    int n = (int) igraph_matrix_nrow(a);
-    int nrhs = (int) igraph_matrix_ncol(b);
-    int lda = n > 0 ? n : 1;
-    int ldb = n > 0 ? n : 1;
+    int n;
+    int nrhs;
+    int lda;
+    int ldb;
     int info;
+    igraph_vector_fortran_int_t vipiv;
+
+    if (igraph_matrix_nrow(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
+    if (igraph_matrix_ncol(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of columns in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
+
+    n = (int) igraph_matrix_nrow(a);
+    nrhs = (int) igraph_matrix_ncol(b);
+    lda = n > 0 ? n : 1;
+    ldb = n > 0 ? n : 1;
 
     if (n != igraph_matrix_ncol(a)) {
         IGRAPH_ERROR("Cannot LU solve matrix.", IGRAPH_NONSQUARE);
@@ -155,9 +223,17 @@ int igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix_t *a,
     if (igraph_vector_int_size(ipiv) != n) {
         IGRAPH_ERROR("Pivot vector length must match number of matrix rows.", IGRAPH_EINVAL);
     }
-    igraphdgetrs_(&trans, &n, &nrhs, VECTOR(a->data), &lda, VECTOR(*ipiv),
+
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&vipiv, igraph_vector_int_size(ipiv)));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &vipiv);
+    IGRAPH_CHECK(igraph_vector_int_copy_to_fortran(ipiv, &vipiv));
+
+    igraphdgetrs_(&trans, &n, &nrhs, VECTOR(a->data), &lda, VECTOR(vipiv),
                   VECTOR(b->data), &ldb, &info);
 
+    igraph_vector_fortran_int_destroy(&vipiv);
+    IGRAPH_FINALLY_CLEAN(1);
+
     if (info < 0) {
         switch (info) {
         case -1:
@@ -193,7 +269,7 @@ int igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix_t *a,
         }
     }
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -231,14 +307,20 @@ int igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix_t *a,
  * \example examples/simple/igraph_lapack_dgesv.c
  */
 
-int igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
-                        igraph_matrix_t *b, int *info) {
+igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
+                                   igraph_matrix_t *b, int *info) {
 
+    if (igraph_matrix_nrow(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
+    if (igraph_matrix_ncol(a) > INT_MAX) {
+        IGRAPH_ERROR("Number of columns in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
     int n = (int) igraph_matrix_nrow(a);
     int nrhs = (int) igraph_matrix_ncol(b);
     int lda = n > 0 ? n : 1;
     int ldb = n > 0 ? n : 1;
-    igraph_vector_int_t *myipiv = ipiv, vipiv;
+    igraph_vector_fortran_int_t vipiv;
 
     if (n != igraph_matrix_ncol(a)) {
         IGRAPH_ERROR("Cannot LU solve matrix.", IGRAPH_NONSQUARE);
@@ -247,13 +329,10 @@ int igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
         IGRAPH_ERROR("Cannot LU solve matrix, RHS of wrong size.", IGRAPH_EINVAL);
     }
 
-    if (!ipiv) {
-        IGRAPH_CHECK(igraph_vector_int_init(&vipiv, n));
-        IGRAPH_FINALLY(igraph_vector_int_destroy, &vipiv);
-        myipiv = &vipiv;
-    }
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&vipiv, n));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &vipiv);
 
-    igraphdgesv_(&n, &nrhs, VECTOR(a->data), &lda, VECTOR(*myipiv),
+    igraphdgesv_(&n, &nrhs, VECTOR(a->data), &lda, VECTOR(vipiv),
                  VECTOR(b->data), &ldb, info);
 
     if (*info > 0) {
@@ -290,12 +369,14 @@ int igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
         }
     }
 
-    if (!ipiv) {
-        igraph_vector_int_destroy(&vipiv);
-        IGRAPH_FINALLY_CLEAN(1);
+    if (ipiv) {
+        IGRAPH_CHECK(igraph_vector_int_update_from_fortran(ipiv, &vipiv));
     }
 
-    return 0;
+    igraph_vector_fortran_int_destroy(&vipiv);
+    IGRAPH_FINALLY_CLEAN(1);
+
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -358,7 +439,7 @@ int igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv,
  * \example examples/simple/igraph_lapack_dsyevr.c
  */
 
-int igraph_lapack_dsyevr(const igraph_matrix_t *A,
+igraph_error_t igraph_lapack_dsyevr(const igraph_matrix_t *A,
                          igraph_lapack_dsyev_which_t which,
                          igraph_real_t vl, igraph_real_t vu, int vestimate,
                          int il, int iu, igraph_real_t abstol,
@@ -367,12 +448,15 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
 
     igraph_matrix_t Acopy;
     char jobz = vectors ? 'V' : 'N', range, uplo = 'U';
+    if (igraph_matrix_nrow(A) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
     int n = (int) igraph_matrix_nrow(A), lda = n, ldz = n;
     int m, info;
     igraph_vector_t *myvalues = values, vvalues;
-    igraph_vector_int_t *mysupport = support, vsupport;
+    igraph_vector_fortran_int_t mysupport;
     igraph_vector_t work;
-    igraph_vector_int_t iwork;
+    igraph_vector_fortran_int_t iwork;
     int lwork = -1, liwork = -1;
 
     if (n != igraph_matrix_ncol(A)) {
@@ -387,43 +471,41 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
         IGRAPH_ERROR("Invalid 'il' and/or 'iu' values.", IGRAPH_EINVAL);
     }
 
-    IGRAPH_CHECK(igraph_matrix_copy(&Acopy, A));
+    IGRAPH_CHECK(igraph_matrix_init_copy(&Acopy, A));
     IGRAPH_FINALLY(igraph_matrix_destroy, &Acopy);
 
     IGRAPH_VECTOR_INIT_FINALLY(&work, 1);
-    IGRAPH_CHECK(igraph_vector_int_init(&iwork, 1));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &iwork);
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&iwork, 1));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &iwork);
 
     if (!values) {
         IGRAPH_VECTOR_INIT_FINALLY(&vvalues, 0);
         myvalues = &vvalues;
     }
-    if (!support) {
-        IGRAPH_CHECK(igraph_vector_int_init(&vsupport, 0));
-        IGRAPH_FINALLY(igraph_vector_int_destroy, &vsupport);
-        mysupport = &vsupport;
-    }
+
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&mysupport, 0));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &mysupport);
 
     IGRAPH_CHECK(igraph_vector_resize(myvalues, n));
 
     switch (which) {
     case IGRAPH_LAPACK_DSYEV_ALL:
         range = 'A';
-        IGRAPH_CHECK(igraph_vector_int_resize(mysupport, 2 * n));
+        IGRAPH_CHECK(igraph_vector_fortran_int_resize(&mysupport, 2 * n));
         if (vectors) {
             IGRAPH_CHECK(igraph_matrix_resize(vectors, n, n));
         }
         break;
     case IGRAPH_LAPACK_DSYEV_INTERVAL:
         range = 'V';
-        IGRAPH_CHECK(igraph_vector_int_resize(mysupport, 2 * vestimate));
+        IGRAPH_CHECK(igraph_vector_fortran_int_resize(&mysupport, 2 * vestimate));
         if (vectors) {
             IGRAPH_CHECK(igraph_matrix_resize(vectors, n, vestimate));
         }
         break;
     case IGRAPH_LAPACK_DSYEV_SELECT:
         range = 'I';
-        IGRAPH_CHECK(igraph_vector_int_resize(mysupport, 2 * (iu - il + 1)));
+        IGRAPH_CHECK(igraph_vector_fortran_int_resize(&mysupport, 2 * (iu - il + 1)));
         if (vectors) {
             IGRAPH_CHECK(igraph_matrix_resize(vectors, n, iu - il + 1));
         }
@@ -432,7 +514,7 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
 
     igraphdsyevr_(&jobz, &range, &uplo, &n, &MATRIX(Acopy, 0, 0), &lda,
                   &vl, &vu, &il, &iu, &abstol, &m, VECTOR(*myvalues),
-                  vectors ? &MATRIX(*vectors, 0, 0) : 0, &ldz, VECTOR(*mysupport),
+                  vectors ? &MATRIX(*vectors, 0, 0) : 0, &ldz, VECTOR(mysupport),
                   VECTOR(work), &lwork, VECTOR(iwork), &liwork, &info);
 
     if (info != 0) {
@@ -442,11 +524,11 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
     lwork = (int) VECTOR(work)[0];
     liwork = VECTOR(iwork)[0];
     IGRAPH_CHECK(igraph_vector_resize(&work, lwork));
-    IGRAPH_CHECK(igraph_vector_int_resize(&iwork, liwork));
+    IGRAPH_CHECK(igraph_vector_fortran_int_resize(&iwork, liwork));
 
     igraphdsyevr_(&jobz, &range, &uplo, &n, &MATRIX(Acopy, 0, 0), &lda,
                   &vl, &vu, &il, &iu, &abstol, &m, VECTOR(*myvalues),
-                  vectors ? &MATRIX(*vectors, 0, 0) : 0, &ldz, VECTOR(*mysupport),
+                  vectors ? &MATRIX(*vectors, 0, 0) : 0, &ldz, VECTOR(mysupport),
                   VECTOR(work), &lwork, VECTOR(iwork), &liwork, &info);
 
     if (info != 0) {
@@ -460,24 +542,24 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
         IGRAPH_CHECK(igraph_matrix_resize(vectors, n, m));
     }
     if (support) {
+        IGRAPH_CHECK(igraph_vector_int_update_from_fortran(support, &mysupport));
         IGRAPH_CHECK(igraph_vector_int_resize(support, m));
     }
 
-    if (!support) {
-        igraph_vector_int_destroy(&vsupport);
-        IGRAPH_FINALLY_CLEAN(1);
-    }
+    igraph_vector_fortran_int_destroy(&mysupport);
+    IGRAPH_FINALLY_CLEAN(1);
+
     if (!values) {
         igraph_vector_destroy(&vvalues);
         IGRAPH_FINALLY_CLEAN(1);
     }
 
-    igraph_vector_int_destroy(&iwork);
+    igraph_vector_fortran_int_destroy(&iwork);
     igraph_vector_destroy(&work);
     igraph_matrix_destroy(&Acopy);
     IGRAPH_FINALLY_CLEAN(3);
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -538,7 +620,7 @@ int igraph_lapack_dsyevr(const igraph_matrix_t *A,
  * \example examples/simple/igraph_lapack_dgeev.c
  */
 
-int igraph_lapack_dgeev(const igraph_matrix_t *A,
+igraph_error_t igraph_lapack_dgeev(const igraph_matrix_t *A,
                         igraph_vector_t *valuesreal,
                         igraph_vector_t *valuesimag,
                         igraph_matrix_t *vectorsleft,
@@ -547,6 +629,11 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
 
     char jobvl = vectorsleft  ? 'V' : 'N';
     char jobvr = vectorsright ? 'V' : 'N';
+    igraph_real_t dummy;   /* to prevent some Clang sanitizer warnings */
+
+    if (igraph_matrix_nrow(A) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
     int n = (int) igraph_matrix_nrow(A);
     int lda = n, ldvl = n, ldvr = n, lwork = -1;
     igraph_vector_t work;
@@ -558,7 +645,7 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
         IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_NONSQUARE);
     }
 
-    IGRAPH_CHECK(igraph_matrix_copy(&Acopy, A));
+    IGRAPH_CHECK(igraph_matrix_init_copy(&Acopy, A));
     IGRAPH_FINALLY(igraph_matrix_destroy, &Acopy);
 
     IGRAPH_VECTOR_INIT_FINALLY(&work, 1);
@@ -584,8 +671,8 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
 
     igraphdgeev_(&jobvl, &jobvr, &n, &MATRIX(Acopy, 0, 0), &lda,
                  VECTOR(*myreal), VECTOR(*myimag),
-                 vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : 0, &ldvl,
-                 vectorsright ? &MATRIX(*vectorsright, 0, 0) : 0, &ldvr,
+                 vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : &dummy, &ldvl,
+                 vectorsright ? &MATRIX(*vectorsright, 0, 0) : &dummy, &ldvr,
                  VECTOR(work), &lwork, info);
 
     lwork = (int) VECTOR(work)[0];
@@ -593,8 +680,8 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
 
     igraphdgeev_(&jobvl, &jobvr, &n, &MATRIX(Acopy, 0, 0), &lda,
                  VECTOR(*myreal), VECTOR(*myimag),
-                 vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : 0, &ldvl,
-                 vectorsright ? &MATRIX(*vectorsright, 0, 0) : 0, &ldvr,
+                 vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : &dummy, &ldvl,
+                 vectorsright ? &MATRIX(*vectorsright, 0, 0) : &dummy, &ldvr,
                  VECTOR(work), &lwork, info);
 
     if (*info < 0) {
@@ -620,7 +707,7 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
     igraph_matrix_destroy(&Acopy);
     IGRAPH_FINALLY_CLEAN(2);
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -741,7 +828,7 @@ int igraph_lapack_dgeev(const igraph_matrix_t *A,
  * \example examples/simple/igraph_lapack_dgeevx.c
  */
 
-int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
+igraph_error_t igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
                          const igraph_matrix_t *A,
                          igraph_vector_t *valuesreal,
                          igraph_vector_t *valuesimag,
@@ -757,14 +844,18 @@ int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
     char jobvl = vectorsleft  ? 'V' : 'N';
     char jobvr = vectorsright ? 'V' : 'N';
     char sense;
+    if (igraph_matrix_nrow(A) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
     int n = (int) igraph_matrix_nrow(A);
     int lda = n, ldvl = n, ldvr = n, lwork = -1;
     igraph_vector_t work;
-    igraph_vector_int_t iwork;
+    igraph_vector_fortran_int_t iwork;
     igraph_matrix_t Acopy;
     int error = *info;
     igraph_vector_t *myreal = valuesreal, *myimag = valuesimag, vreal, vimag;
     igraph_vector_t *myscale = scale, vscale;
+    igraph_real_t dummy;   /* to prevent some Clang sanitizer warnings */
 
     if (igraph_matrix_ncol(A) != n) {
         IGRAPH_ERROR("Cannot calculate eigenvalues (dgeevx).", IGRAPH_NONSQUARE);
@@ -798,12 +889,12 @@ int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
         sense = 'B';
     }
 
-    IGRAPH_CHECK(igraph_matrix_copy(&Acopy, A));
+    IGRAPH_CHECK(igraph_matrix_init_copy(&Acopy, A));
     IGRAPH_FINALLY(igraph_matrix_destroy, &Acopy);
 
     IGRAPH_VECTOR_INIT_FINALLY(&work, 1);
-    IGRAPH_CHECK(igraph_vector_int_init(&iwork, n));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &iwork);
+    IGRAPH_CHECK(igraph_vector_fortran_int_init(&iwork, n));
+    IGRAPH_FINALLY(igraph_vector_fortran_int_destroy, &iwork);
 
     if (!valuesreal) {
         IGRAPH_VECTOR_INIT_FINALLY(&vreal, n);
@@ -832,11 +923,11 @@ int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
 
     igraphdgeevx_(&balanc, &jobvl, &jobvr, &sense, &n, &MATRIX(Acopy, 0, 0),
                   &lda, VECTOR(*myreal), VECTOR(*myimag),
-                  vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : 0, &ldvl,
-                  vectorsright ? &MATRIX(*vectorsright, 0, 0) : 0, &ldvr,
+                  vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : &dummy, &ldvl,
+                  vectorsright ? &MATRIX(*vectorsright, 0, 0) : &dummy, &ldvr,
                   ilo, ihi, VECTOR(*myscale), abnrm,
-                  rconde ? VECTOR(*rconde) : 0,
-                  rcondv ? VECTOR(*rcondv) : 0,
+                  rconde ? VECTOR(*rconde) : &dummy,
+                  rcondv ? VECTOR(*rcondv) : &dummy,
                   VECTOR(work), &lwork, VECTOR(iwork), info);
 
     lwork = (int) VECTOR(work)[0];
@@ -844,11 +935,11 @@ int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
 
     igraphdgeevx_(&balanc, &jobvl, &jobvr, &sense, &n, &MATRIX(Acopy, 0, 0),
                   &lda, VECTOR(*myreal), VECTOR(*myimag),
-                  vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : 0, &ldvl,
-                  vectorsright ? &MATRIX(*vectorsright, 0, 0) : 0, &ldvr,
+                  vectorsleft  ? &MATRIX(*vectorsleft, 0, 0) : &dummy, &ldvl,
+                  vectorsright ? &MATRIX(*vectorsright, 0, 0) : &dummy, &ldvr,
                   ilo, ihi, VECTOR(*myscale), abnrm,
-                  rconde ? VECTOR(*rconde) : 0,
-                  rcondv ? VECTOR(*rcondv) : 0,
+                  rconde ? VECTOR(*rconde) : &dummy,
+                  rcondv ? VECTOR(*rcondv) : &dummy,
                   VECTOR(work), &lwork, VECTOR(iwork), info);
 
     if (*info < 0) {
@@ -876,18 +967,21 @@ int igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
         IGRAPH_FINALLY_CLEAN(1);
     }
 
-    igraph_vector_int_destroy(&iwork);
+    igraph_vector_fortran_int_destroy(&iwork);
     igraph_vector_destroy(&work);
     igraph_matrix_destroy(&Acopy);
     IGRAPH_FINALLY_CLEAN(3);
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
-int igraph_lapack_dgehrd(const igraph_matrix_t *A,
+igraph_error_t igraph_lapack_dgehrd(const igraph_matrix_t *A,
                          int ilo, int ihi,
                          igraph_matrix_t *result) {
 
+    if (igraph_matrix_nrow(A) > INT_MAX) {
+        IGRAPH_ERROR("Number of rows in matrix too large for LAPACK.", IGRAPH_EOVERFLOW);
+    }
     int n = (int) igraph_matrix_nrow(A);
     int lda = n;
     int lwork = -1;
@@ -908,10 +1002,10 @@ int igraph_lapack_dgehrd(const igraph_matrix_t *A,
 
     if (n <= 1) {
         IGRAPH_CHECK(igraph_matrix_update(result, A));
-        return 0;
+        return IGRAPH_SUCCESS;
     }
 
-    IGRAPH_CHECK(igraph_matrix_copy(&Acopy, A));
+    IGRAPH_CHECK(igraph_matrix_init_copy(&Acopy, A));
     IGRAPH_FINALLY(igraph_matrix_destroy, &Acopy);
     IGRAPH_VECTOR_INIT_FINALLY(&tau, n - 1);
 
@@ -950,7 +1044,5 @@ int igraph_lapack_dgehrd(const igraph_matrix_t *A,
         }
     }
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
-
-
