@@ -52,25 +52,25 @@
  * \param res Pointer to a real number, the result will be stored
  *   here.
  * \param loops Logical constant, whether to include self-loops in the
- *   calculation. If this constant is TRUE then
+ *   calculation. If this constant is \c true then
  *   loop edges are thought to be possible in the graph (this does not
  *   necessarily mean that the graph really contains any loops). If
- *   this is FALSE then the result is only correct if the graph does not
+ *   this is \c false then the result is only correct if the graph does not
  *   contain loops.
  * \return Error code.
  *
  * Time complexity: O(1).
  */
-int igraph_density(const igraph_t *graph, igraph_real_t *res,
+igraph_error_t igraph_density(const igraph_t *graph, igraph_real_t *res,
                    igraph_bool_t loops) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_real_t no_of_edges = igraph_ecount(graph);
+    igraph_real_t no_of_nodes = (igraph_real_t) igraph_vcount(graph);
+    igraph_real_t no_of_edges = (igraph_real_t) igraph_ecount(graph);
     igraph_bool_t directed = igraph_is_directed(graph);
 
     if (no_of_nodes == 0) {
         *res = IGRAPH_NAN;
-        return 0;
+        return IGRAPH_SUCCESS;
     }
 
     if (!loops) {
@@ -89,12 +89,12 @@ int igraph_density(const igraph_t *graph, igraph_real_t *res,
         }
     }
 
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 /**
  * \function igraph_diversity
- * Structural diversity index of the vertices
+ * \brief Structural diversity index of the vertices.
  *
  * This measure was defined in Nathan Eagle, Michael Macy and Rob
  * Claxton: Network Diversity and Economic Development, Science 328,
@@ -107,7 +107,8 @@ int igraph_density(const igraph_t *graph, igraph_real_t *res,
  * where p[i,j]=w[i,j]/sum(w[i,l], l=1..k[i]),  k[i] is the (total)
  * degree of vertex i, and w[i,j] is the weight of the edge(s) between
  * vertex i and j. The diversity of isolated vertices will be NaN
- * (not-a-number).
+ * (not-a-number), while that of vertices with a single connection
+ * will be zero.
  *
  * </para><para>
  * The measure works only if the graph is undirected and has no multiple edges.
@@ -116,8 +117,8 @@ int igraph_density(const igraph_t *graph, igraph_real_t *res,
  * graph with \ref igraph_to_undirected() .
  *
  * \param graph The undirected input graph.
- * \param weights The edge weights, in the order of the edge ids, must
- *    have appropriate length.
+ * \param weights The edge weights, in the order of the edge IDs, must
+ *    have appropriate length. Weights must be non-negative.
  * \param res An initialized vector, the results are stored here.
  * \param vids Vertex selector that specifies the vertices which to calculate
  *    the measure.
@@ -126,16 +127,14 @@ int igraph_density(const igraph_t *graph, igraph_real_t *res,
  * Time complexity: O(|V|+|E|), linear.
  *
  */
-int igraph_diversity(const igraph_t *graph, const igraph_vector_t *weights,
+igraph_error_t igraph_diversity(const igraph_t *graph, const igraph_vector_t *weights,
                      igraph_vector_t *res, const igraph_vs_t vids) {
 
-    int no_of_nodes = igraph_vcount(graph);
-    int no_of_edges = igraph_ecount(graph);
-    igraph_vector_t incident;
-    igraph_vit_t vit;
-    igraph_real_t s, ent, w;
-    int i, j, k;
+    igraph_integer_t no_of_edges = igraph_ecount(graph);
+    igraph_integer_t k, i;
+    igraph_vector_int_t incident;
     igraph_bool_t has_multiple;
+    igraph_vit_t vit;
 
     if (igraph_is_directed(graph)) {
         IGRAPH_ERROR("Diversity measure works with undirected graphs only.", IGRAPH_EINVAL);
@@ -154,48 +153,65 @@ int igraph_diversity(const igraph_t *graph, const igraph_vector_t *weights,
         IGRAPH_ERROR("Diversity measure works only if the graph has no multiple edges.", IGRAPH_EINVAL);
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&incident, 10);
-
-    if (igraph_vs_is_all(&vids)) {
-        IGRAPH_CHECK(igraph_vector_resize(res, no_of_nodes));
-        for (i = 0; i < no_of_nodes; i++) {
-            s = ent = 0.0;
-            IGRAPH_CHECK(igraph_incident(graph, &incident, i, /*mode=*/ IGRAPH_ALL));
-            for (j = 0, k = (int) igraph_vector_size(&incident); j < k; j++) {
-                w = VECTOR(*weights)[(long int)VECTOR(incident)[j]];
-                s += w;
-                ent += (w * log(w));
-            }
-            VECTOR(*res)[i] = (log(s) - ent / s) / log(k);
+    if (no_of_edges > 0) {
+        igraph_real_t minweight = igraph_vector_min(weights);
+        if (minweight < 0) {
+            IGRAPH_ERROR("Weight vector must be non-negative.", IGRAPH_EINVAL);
+        } else if (igraph_is_nan(minweight)) {
+            IGRAPH_ERROR("Weight vector must not contain NaN values.", IGRAPH_EINVAL);
         }
-    } else {
-        IGRAPH_CHECK(igraph_vector_resize(res, 0));
-        IGRAPH_CHECK(igraph_vit_create(graph, vids, &vit));
-        IGRAPH_FINALLY(igraph_vit_destroy, &vit);
-
-        for (IGRAPH_VIT_RESET(vit), i = 0;
-             !IGRAPH_VIT_END(vit);
-             IGRAPH_VIT_NEXT(vit), i++) {
-            long int v = IGRAPH_VIT_GET(vit);
-            s = ent = 0.0;
-            IGRAPH_CHECK(igraph_incident(graph, &incident, (igraph_integer_t) v,
-                                         /*mode=*/ IGRAPH_ALL));
-            for (j = 0, k = (int) igraph_vector_size(&incident); j < k; j++) {
-                w = VECTOR(*weights)[(long int)VECTOR(incident)[j]];
-                s += w;
-                ent += (w * log(w));
-            }
-            IGRAPH_CHECK(igraph_vector_push_back(res, (log(s) - ent / s) / log(k)));
-        }
-
-        igraph_vit_destroy(&vit);
-        IGRAPH_FINALLY_CLEAN(1);
     }
 
-    igraph_vector_destroy(&incident);
-    IGRAPH_FINALLY_CLEAN(1);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&incident, 10);
 
-    return 0;
+    IGRAPH_CHECK(igraph_vit_create(graph, vids, &vit));
+    IGRAPH_FINALLY(igraph_vit_destroy, &vit);
+
+    igraph_vector_clear(res);
+    IGRAPH_CHECK(igraph_vector_reserve(res, IGRAPH_VIT_SIZE(vit)));
+
+    for (IGRAPH_VIT_RESET(vit); !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit)) {
+        igraph_real_t d;
+        igraph_integer_t v = IGRAPH_VIT_GET(vit);
+
+        IGRAPH_CHECK(igraph_incident(graph, &incident, v, /*mode=*/ IGRAPH_ALL));
+        k = igraph_vector_int_size(&incident); /* degree */
+
+        /*
+         * Non-normalized diversity is defined as
+         * d = -sum_i w_i/s log (w_i/s)
+         * where s = sum_i w_i. In order to avoid two passes through the w vector,
+         * we use the equivalent formulation of
+         * d = log s - (sum_i w_i log w_i) / s
+         * However, this formulation may not give an exact 0.0 for some w when k=1,
+         * due to roundoff errors (examples: w=3 or w=7). For this reason, we
+         * special-case the computation for k=1 even for the unnormalized diversity
+         * insted of just setting the normalization factor to 1 for this case.
+         */
+        if (k == 0) {
+            d = IGRAPH_NAN;
+        } else if (k == 1) {
+            if (VECTOR(*weights)[0] > 0) d = 0.0; /* s > 0 */
+            else d = IGRAPH_NAN; /* s == 0 */
+        } else {
+            igraph_real_t s = 0.0, ent = 0.0;
+            for (i = 0; i < k; i++) {
+                igraph_real_t w = VECTOR(*weights)[VECTOR(incident)[i]];
+                if (w == 0) continue;
+                s += w;
+                ent += (w * log(w));
+            }
+            d = (log(s) - ent / s) / log(k);
+        }
+
+        igraph_vector_push_back(res, d); /* reserved */
+    }
+
+    igraph_vit_destroy(&vit);
+    igraph_vector_int_destroy(&incident);
+    IGRAPH_FINALLY_CLEAN(2);
+
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -241,37 +257,37 @@ int igraph_diversity(const igraph_t *graph, const igraph_vector_t *weights,
  *
  * \example examples/simple/igraph_reciprocity.c
  */
-int igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
+igraph_error_t igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
                        igraph_bool_t ignore_loops,
                        igraph_reciprocity_t mode) {
 
     igraph_integer_t nonrec = 0, rec = 0, loops = 0;
-    igraph_vector_t inneis, outneis;
-    long int i;
-    long int no_of_nodes = igraph_vcount(graph);
+    igraph_vector_int_t inneis, outneis;
+    igraph_integer_t i;
+    igraph_integer_t no_of_nodes = igraph_vcount(graph);
 
     if (mode != IGRAPH_RECIPROCITY_DEFAULT &&
         mode != IGRAPH_RECIPROCITY_RATIO) {
-        IGRAPH_ERROR("Invalid reciprocity type", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Invalid reciprocity type.", IGRAPH_EINVAL);
     }
 
     /* THIS IS AN EXIT HERE !!!!!!!!!!!!!! */
     if (!igraph_is_directed(graph)) {
         *res = 1.0;
-        return 0;
+        return IGRAPH_SUCCESS;
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&inneis, 0);
-    IGRAPH_VECTOR_INIT_FINALLY(&outneis, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&inneis, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&outneis, 0);
 
     for (i = 0; i < no_of_nodes; i++) {
-        long int ip, op;
-        igraph_neighbors(graph, &inneis, (igraph_integer_t) i, IGRAPH_IN);
-        igraph_neighbors(graph, &outneis, (igraph_integer_t) i, IGRAPH_OUT);
+        igraph_integer_t ip, op;
+        IGRAPH_CHECK(igraph_neighbors(graph, &inneis, i, IGRAPH_IN));
+        IGRAPH_CHECK(igraph_neighbors(graph, &outneis, i, IGRAPH_OUT));
 
         ip = op = 0;
-        while (ip < igraph_vector_size(&inneis) &&
-               op < igraph_vector_size(&outneis)) {
+        while (ip < igraph_vector_int_size(&inneis) &&
+               op < igraph_vector_int_size(&outneis)) {
             if (VECTOR(inneis)[ip] < VECTOR(outneis)[op]) {
                 nonrec += 1;
                 ip++;
@@ -294,8 +310,8 @@ int igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
                 op++;
             }
         }
-        nonrec += (igraph_vector_size(&inneis) - ip) +
-                  (igraph_vector_size(&outneis) - op);
+        nonrec += (igraph_vector_int_size(&inneis) - ip) +
+                  (igraph_vector_int_size(&outneis) - op);
     }
 
     if (mode == IGRAPH_RECIPROCITY_DEFAULT) {
@@ -308,8 +324,8 @@ int igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
         *res = (igraph_real_t) rec / (rec + nonrec);
     }
 
-    igraph_vector_destroy(&inneis);
-    igraph_vector_destroy(&outneis);
+    igraph_vector_int_destroy(&inneis);
+    igraph_vector_int_destroy(&outneis);
     IGRAPH_FINALLY_CLEAN(2);
-    return 0;
+    return IGRAPH_SUCCESS;
 }
